
/** @file Ssum_to_Infinity.cc
 *
 * Implementation of Ssums to Infinity
 *
 */

/*
 *  gTybalt Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <stdexcept>
#include "nestedsums.h"

#ifdef _DEBUG
#include <ctime>
#endif // def _DEBUG

namespace GiNaC {

GINAC_IMPLEMENT_REGISTERED_CLASS(Ssum_to_Infinity, Ssum)



  Ssum_to_Infinity::Ssum_to_Infinity() : inherited(TINFO_Ssum_to_Infinity,Infinity,_empty_list)
  { 
  }

  Ssum_to_Infinity::Ssum_to_Infinity(unsigned ti) : inherited(ti,Infinity,_empty_list)
  { 
  }

  void Ssum_to_Infinity::destroy(bool call_parent)
  {
    if (call_parent)
      inherited::destroy(call_parent);
  }

  void Ssum_to_Infinity::copy(const Ssum_to_Infinity &other)
  {
    inherited::copy(other);
  }

  void Ssum_to_Infinity::archive(archive_node &node) const
  {
    inherited::archive(node);
  }

  Ssum_to_Infinity::Ssum_to_Infinity(const archive_node &node, const lst &sym_lst) : inherited(node, sym_lst)
  {
  }

  ex Ssum_to_Infinity::unarchive(const archive_node &node, const lst &sym_lst)
  {
    return (new Ssum_to_Infinity(node, sym_lst))->setflag(status_flags::dynallocated);
  }

  int Ssum_to_Infinity::compare_same_type(const basic &other) const
  {
    const Ssum_to_Infinity &o = dynamic_cast<const Ssum_to_Infinity &>(other);
    int cmpval;

    cmpval = letter_list.compare(o.letter_list);
    if ( cmpval) return cmpval;

    return n.compare(o.n);
  }

  // other ctors

  // ctor with initialization list
  Ssum_to_Infinity::Ssum_to_Infinity(const ex & llc) : inherited(TINFO_Ssum_to_Infinity,Infinity,llc)
  { }

  Ssum_to_Infinity::Ssum_to_Infinity(unsigned ti, const ex & llc) : inherited(ti,Infinity,llc)
  { }

  // functions overriding virtual functions from bases classes

  /**
   *
   * The simplifications are done in the following order:
   *
   * - If all \f$x_j\f$'s are equal to 1, we have a harmonic sum to infinity.
   *
   */
  ex Ssum_to_Infinity::eval(int level) const
  {
    unsigned depth = get_depth();

    // check if simplifies to harmonic sum to Infinity
    if ( depth==0 ) return create_harmonic_sum_to_Infinity(letter_list);
    int i=0;
    while ( is_a<unit_letter>(letter_list.op(i++).eval()) )
      {
	if ( i==depth) return create_harmonic_sum_to_Infinity(letter_list);
      }

    return this->hold();

  }

  // static member variables

  // same value as class power
  unsigned Ssum_to_Infinity::precedence = 60;

} // namespace GiNaC

