
/** @file helpers.h
 *
 * Interface to some helpers
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_HELPERS_H__
#define __NESTEDSUMS_HELPERS_H__

#include "ginac/ginac.h"

namespace GiNaC {

 ex nestedsums_helper_expand(const ex & expr, int level = 0);
 ex nestedsums_helper_multiply_via_triangle(const ex & A, const ex & B,
                      		            const ex & expansion_parameter, int order);
 ex nestedsums_helper_multiply_via_triangle_speedy(const ex & A, const ex & B,
						   const ex & expansion_parameter, int order);
 ex nestedsums_helper_multiply_laurent(const ex & A, const ex & B, 
                                       const ex & expansion_parameter, int order);
 ex nestedsums_helper_multiply_laurent_speedy(const ex & A, const ex & B, 
					      const ex & expansion_parameter, int order);
 ex nestedsums_helper_expand_tgamma_fct(const ex & a1, const ex & b1, const ex & a2, const ex & b2,
					const ex & expansion_parameter, int order);
 ex nestedsums_helper_expand_lst_tgamma_fct(const ex & ll1, const ex & ll2, 
					    const ex & expansion_parameter, int order);

 ex nestedsums_helper_expand_power(const ex & x, const ex & m, const ex & expansion_parameter, int order);
 ex nestedsums_helper_series(const ex & f, const ex & expansion_parameter, int order);

 ex nestedsums_helper_bernoulli(int n);

 ex nestedsums_helper_lowering_op_geo_0(const ex & x, int m);
 ex nestedsums_helper_lowering_op_geo_1(const ex & x, int m);

 ex nestedsums_helper_arithmetic_sum_type_Z(const ex & n, const ex & m);
 ex nestedsums_helper_arithmetic_sum_type_S(const ex & n, const ex & m);

 int nestedsums_helper_get_degree(const ex & expr, const ex & eps);
 int nestedsums_helper_ldegree(const ex & expr, const ex & eps);
 ex nestedsums_helper_coeff(const ex & expr, const ex & eps, int n = 1);

 exvector reverse_exvector_from_lst(const ex & l);
 exvector exvector_append_lst(const exvector & v, const ex & l);
 exvector exvector_multiply_last_letter(const exvector & v, const ex & x);
 exvector exvector_increase_last_degree(const exvector & v);

 class nestedsums_helper_less
   {
     public :
       bool operator() (const ex & a, const ex & b) const;
   };

 bool is_class_in_nestedsums(const ex & expr);
 bool is_expr_in_sym_lst(const ex & expr, const lst & sym_lst);

 ex nestedsums_helper_normalize(const ex & expr, const lst & sym_lst = _empty_list);
 ex nestedsums_helper_sort(const ex & expr, const lst & sym_lst = _empty_list);

 ex convert_Zsums_to_standard_form(const ex & expr, const lst & sym_lst = _empty_list);

 void nestedsums_helper_clear_hashes(void);

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_HELPERS_H__

