
/** @file letter.h
 *
 * Interface to letter
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_LETTER_H__
#define __NESTEDSUMS_LETTER_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// A letter is a basic_letter with offset=0.
  /**
   *
   *  The model for a letter is
   *  \f[
   *      \frac{x^i}{i^n}
   *  \f]
   *  Here \f$x\f$ is the scale, \f$n\f$ is the degree 
   *  and \f$i\f$ the index.
   *
   */
  class letter : public basic_letter
    {

      GINAC_DECLARE_REGISTERED_CLASS(letter, basic_letter)

	// ctors
    public:
      explicit letter(unsigned ti);
      letter(const ex & l, const ex & d);
      letter(const ex & l, const ex & d, const ex & i);
      letter(unsigned ti, const ex & l, const ex & d);
      letter(unsigned ti, const ex & l, const ex & d, const ex & i);

	// functions overriding virtual functions from base classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;

      ex concat_speedy(const ex & l) const;

	// non-virtual functions 

	// member variables :

    protected:

      static unsigned precedence;
    };

  // utility functions
inline const letter &ex_to_letter(const ex &e)
  {
    return static_cast<const letter &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_letter(const ex & l, const ex & d)
  {
    return (new letter(l,d))->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_letter(const ex & l, const ex & d, const ex & i)
  {
    return (new letter(l,d,i))->setflag(status_flags::dynallocated);
  }

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_LETTER_H__
