
/** @file multiple_polylog.h
 *
 * Interface to multiple polylog sums 
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_MULTIPLE_POLYLOG_H__
#define __NESTEDSUMS_MULTIPLE_POLYLOG_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// A multiple polylog sum is a special case of a Zsum.
  /**
   *
   *  Multiple polylogs are defined by
   *  \f[
   *     \mbox{Li}_{m_k,...,m_1}(x_k,...,x_1) = Z(\infty;m_1,...,m_k;x_1,...,x_k) 
   *  \f]
   *
   */
  class multiple_polylog : public virtual Zsum
    {
      GINAC_DECLARE_REGISTERED_CLASS(multiple_polylog, Zsum)

	// ctors
    public:
      explicit multiple_polylog(unsigned ti);
      explicit multiple_polylog(const ex & llc);
      multiple_polylog(unsigned ti, const ex & llc);

	// functions overriding virtual functions from bases classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;

    protected:
      unsigned return_type(void) const { return return_types::noncommutative; }
      unsigned return_type_tinfo(void) const { return TINFO_multiple_polylog; }

      // new virtual functions
    public:
      virtual ex eval_approx(int level = 0) const;

	// member variables :

    protected:

      static unsigned precedence;
    };

  // utility functions
inline const multiple_polylog &ex_to_multiple_polylog(const ex &e)
  {
    return dynamic_cast<const multiple_polylog &>(*e.bp);
  }

  // template specialization
template<> inline const multiple_polylog &ex_to<multiple_polylog>(const ex &e)
  {
    return dynamic_cast<const multiple_polylog &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_multiple_polylog(const ex & llc)
  {
    return (new multiple_polylog(llc))->setflag(status_flags::dynallocated);
  }

 ex eval_multiple_polylog_approx(const ex & expr);

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_MULTIPLE_POLYLOG_H__
