
/** @file unit_letter.h
 *
 * Interface to unit_letter
 *
 */

/*
 *  Copyright (C) 2001 Stefan Weinzierl
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NESTEDSUMS_UNIT_LETTER_H__
#define __NESTEDSUMS_UNIT_LETTER_H__

#include "ginac/ginac.h"

namespace GiNaC {

  /// A unit_letter is a letter whose letter content equals 1.
  /**
   *
   *  The model for a unit_letter is
   *  \f[
   *      \frac{1}{i^n}
   *  \f]
   *  Here \f$n\f$ is the degree 
   *  and \f$i\f$ the summation index.
   *
   */
  class unit_letter : public letter
    {
      GINAC_DECLARE_REGISTERED_CLASS(unit_letter, letter)

	// ctors
    public:
      explicit unit_letter(unsigned ti);
      explicit unit_letter(const ex & d);
      unit_letter(const ex & d, const ex & i);
      unit_letter(unsigned ti, const ex & d);
      unit_letter(unsigned ti, const ex & d, const ex & i);

	// functions overriding virtual functions from base classes
    public:
      void print(const print_context & c, unsigned level = 0) const;
      ex eval(int level = 0) const;

      ex concat_speedy(const ex & l) const;

	// member variables :

      static unsigned precedence;
    };

  // utility functions
inline const unit_letter &ex_to_unit_letter(const ex &e)
  {
    return static_cast<const unit_letter &>(*e.bp);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_unit_letter(const ex & d)
  {
    return (new unit_letter(d))->setflag(status_flags::dynallocated);
  }

  /**
   *
   * Named ctor on the heap.
   *
   */
inline ex create_unit_letter(const ex & d, const ex & i)
  {
    return (new unit_letter(d,i))->setflag(status_flags::dynallocated);
  }

} // namespace GiNaC

#endif // ndef __NESTEDSUMS_UNIT_LETTER_H__
